@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -109,6 +109,8 @@
 #include <libutil.h>
 #endif
 
+#include <uim/uim.h>
+
 #include "udsock.h"
 #include "str.h"
 #include "uim-fep.h"
@@ -193,7 +195,7 @@ static void init_uim(const char *engine)
     printf("uim_init error\n");
     exit(EXIT_FAILURE);
   }
-  g_context = uim_create_context(NULL, get_enc(), NULL, engine, uim_iconv, commit_cb);
+  g_context = uim_create_context(NULL, get_enc(), NULL, engine, NULL, commit_cb);
   nr = uim_get_nr_im(g_context);
   for (i = 0; i < nr; i++) {
     if (strcmp(engine, uim_get_im_name(g_context, i)) == 0) {
@@ -201,7 +203,7 @@ static void init_uim(const char *engine)
     }
   }
   if (i == nr) {
-    printf("%s is not a input method\n\n", engine);
+    printf("%s is not an available input method\n\n", engine);
     usage();
     exit(EXIT_FAILURE);
   }
@@ -218,6 +220,8 @@ static const char *get_default_im_name(void)
   }
 
   engine = uim_get_default_im_name("");  /* instructs native locale */
+
+  /* the string is only valid until next uim API call */
   return engine;
 }
 
@@ -225,10 +229,10 @@ int main(int argc, char **argv)
 {
   /* command will be execed on pty */
   const char **command = malloc(sizeof(const char *) * (argc + 1));
-  const char *engine;
+  char *engine;
   char *sock_path = NULL; /* Socket for backtick */
   int gnu_screen = FALSE;
-  char *env_buf;
+  char pid_str[30];
   struct attribute_tag attr_uim = {
     FALSE,     /* underline */
     FALSE,     /* standout */
@@ -239,10 +243,11 @@ int main(int argc, char **argv)
   };
   FILE *fp;
   const char *suffix = NULL;
-  const char *tmp_dir;
+  char uim_dir[UNIX_PATH_MAX];
   const char *sty_str;
   const char *win_str;
   struct stat stat_buf;
+  const char *errstr;
 
   int op;
 
@@ -255,7 +260,7 @@ int main(int argc, char **argv)
   command[1] = NULL;
 
   init_str();
-  engine = get_default_im_name();
+  engine = strdup(get_default_im_name());
 
   while ((op = getopt(argc, argv, "e:s:u:b:w:t:C:f:SXciodKvh")) != -1) {
     int i;
@@ -306,7 +311,7 @@ int main(int argc, char **argv)
         break;
 
       case 'u':
-        engine = optarg;
+        engine = strdup(optarg);
         break;
 
       case 'c':
@@ -330,7 +335,11 @@ int main(int argc, char **argv)
         break;
 
       case 'w':
-        g_opt.statusline_width = atoi(optarg);
+        g_opt.statusline_width = strtonum(optarg, 1, 10000, &errstr);
+	if (errstr) {
+	  printf("'%s' is %s", optarg, errstr);
+	  return EXIT_FAILURE;
+	}
         if (g_opt.statusline_width <= 0) {
           usage();
           return EXIT_FAILURE;
@@ -406,8 +415,9 @@ opt_end:
   tcgetattr(g_win_in, &s_save_tios);
   setupterm(NULL, g_win_out, NULL);
 
-  if ((tmp_dir = getenv("TMP")) == NULL) {
-    tmp_dir = "/tmp";
+  if (!get_ud_path(uim_dir, sizeof(uim_dir))) {
+    sendline("uim-fep cannot make directory");
+    return EXIT_FAILURE;
   }
 
   sty_str = getenv("STY");
@@ -417,65 +427,62 @@ opt_end:
       puts("STY and WINDOW are not defined");
       return EXIT_FAILURE;
     }
-    snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s-%s", tmp_dir, sty_str, win_str);
-    snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s-%s", tmp_dir, sty_str, win_str);
+    snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/getmode-%s-%s", uim_dir, sty_str, win_str);
+    snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/setmode-%s-%s", uim_dir, sty_str, win_str);
     if (stat(s_path_getmode, &stat_buf) == 0 || stat(s_path_setmode, &stat_buf) == 0) {
       char msg[100];
-      snprintf(msg, 100, "uim-fep is already running on window %s", win_str);
+      snprintf(msg, sizeof(msg), "uim-fep is already running on window %s", win_str);
       sendline(msg);
       return EXIT_FAILURE;
     }
     if (suffix != NULL) {
-      snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s", tmp_dir, suffix);
-      snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s", tmp_dir, suffix);
+      snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/getmode-%s", uim_dir, suffix);
+      snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/setmode-%s", uim_dir, suffix);
     } else {
-      snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s-%s-screen", tmp_dir, sty_str, win_str);
-      snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s-%s-screen", tmp_dir, sty_str, win_str);
+      snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/getmode-%s-%s-screen", uim_dir, sty_str, win_str);
+      snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/setmode-%s-%s-screen", uim_dir, sty_str, win_str);
     }
   } else {
     if (sty_str != NULL && win_str != NULL) {
-      snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s-%s-screen", tmp_dir, sty_str, win_str);
-      snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s-%s-screen", tmp_dir, sty_str, win_str);
+      snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/getmode-%s-%s-screen", uim_dir, sty_str, win_str);
+      snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/setmode-%s-%s-screen", uim_dir, sty_str, win_str);
       if (stat(s_path_getmode, &stat_buf) == 0 || stat(s_path_setmode, &stat_buf) == 0) {
         printf("uim-fep is already running on window %s as filter\n", win_str);
         return EXIT_FAILURE;
       }
       if (suffix != NULL) {
-        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s", tmp_dir, suffix);
-        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s", tmp_dir, suffix);
+        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/getmode-%s", uim_dir, suffix);
+        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/setmode-%s", uim_dir, suffix);
       } else {
-        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s-%s", tmp_dir, sty_str, win_str);
-        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s-%s", tmp_dir, sty_str, win_str);
+        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/getmode-%s-%s", uim_dir, sty_str, win_str);
+        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/setmode-%s-%s", uim_dir, sty_str, win_str);
       }
     } else {
       if (suffix != NULL) {
-        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s", tmp_dir, suffix);
-        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s", tmp_dir, suffix);
+        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/getmode-%s", uim_dir, suffix);
+        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/setmode-%s", uim_dir, suffix);
       } else {
         int file_suffix = 1;
         int pid = getpid();
 
-        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%d", tmp_dir, pid);
-        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%d", tmp_dir, pid);
+        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/getmode-%d", uim_dir, pid);
+        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/setmode-%d", uim_dir, pid);
         while (stat(s_path_getmode, &stat_buf) == 0 || stat(s_path_setmode, &stat_buf) == 0) {
-          snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%d-%d", tmp_dir, pid, file_suffix);
-          snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%d-%d", tmp_dir, pid, file_suffix);
+          snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/getmode-%d-%d", uim_dir, pid, file_suffix);
+          snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/setmode-%d-%d", uim_dir, pid, file_suffix);
           file_suffix++;
         }
       }
     }
   }
 
-  env_buf = malloc(30);
-  snprintf(env_buf, 30, "UIM_FEP_PID=%d", getpid());
-  putenv(env_buf);
+  snprintf(pid_str, sizeof(pid_str), "%d", getpid());
+  setenv("UIM_FEP_PID", pid_str, 1);
 
   if ((fp = fopen(s_path_getmode, "wt")) != NULL) {
     fclose(fp);
     unlink(s_path_getmode);
-    env_buf = malloc(strlen("UIM_FEP_GETMODE=") + strlen(s_path_getmode) + 1);
-    sprintf(env_buf, "UIM_FEP_GETMODE=%s", s_path_getmode);
-    putenv(env_buf);
+    setenv("UIM_FEP_GETMODE", s_path_getmode, 1);
   } else {
     s_path_getmode[0] = '\0';
   }
@@ -483,9 +490,7 @@ opt_end:
   unlink(s_path_setmode);
   if (mkfifo(s_path_setmode, 0600) != -1) {
     unlink(s_path_setmode);
-    env_buf = malloc(strlen("UIM_FEP_SETMODE=") + strlen(s_path_setmode) + 1);
-    sprintf(env_buf, "UIM_FEP_SETMODE=%s", s_path_setmode);
-    putenv(env_buf);
+    setenv("UIM_FEP_SETMODE", s_path_setmode, 1);
   } else {
     s_path_setmode[0] = '\0';
     s_setmode_fd = -1;
@@ -517,6 +522,7 @@ opt_end:
   }
 
   init_uim(engine);
+  free(engine);
   if (gnu_screen) {
     uim_set_mode(g_context, 1);
   }
@@ -668,7 +674,7 @@ static pid_t my_forkpty(int *amaster, struct termios *termp, struct winsize *win
     return pid;
   }
 }
-#elif defined(__svr4__) || defined(__sgi__)
+#elif defined(__svr4__) || defined(__sgi__) || defined(__SVR4)
 static pid_t my_forkpty(int *amaster, struct termios *termp, struct winsize *winp)
 {
   pid_t pid;
@@ -727,6 +733,7 @@ static void main_loop(void)
   int nfd;
   char *_clear_screen = cut_padding(clear_screen);
   char *_clr_eos = cut_padding(clr_eos);
+  const char *errstr;
 
   if (g_win_in > s_master) {
     if (g_win_in > s_setmode_fd) {
@@ -811,13 +818,17 @@ static void main_loop(void)
       int start, end;
 
 #ifdef __CYGWIN32__
-      if ((len = read(s_setmode_fd, buf, sizeof(buf) - 1)) <= 0) {
+      if ((len = read(s_setmode_fd, buf, sizeof(buf) - 1)) == -1 || len == 0) {
         debug2(("pipe closed\n"));
         close(s_setmode_fd);
         s_setmode_fd = open(s_path_setmode, O_RDONLY | O_NONBLOCK);
       }
 #else
-      len = read(s_setmode_fd, buf, sizeof(buf) - 1);
+      if ((len = read(s_setmode_fd, buf, sizeof(buf) - 1)) == -1 || len == 0) {
+        /* XXX: fatal */
+        debug2(("cannot read setmode file\n"));
+        return;
+      }
 #endif
 
       for (end = len - 1; end >= 0 && !isdigit((unsigned char)buf[end]); --end);
@@ -827,8 +838,10 @@ static void main_loop(void)
 
         for (start = end; start > 0 && isdigit((unsigned char)buf[start - 1]); --start);
         buf[end + 1] = '\0';
-        mode = atoi(&buf[start]);
-        if (mode != uim_get_current_mode(g_context)) {
+        mode = strtonum(&buf[start], 0, uim_get_nr_modes(g_context) - 1, &errstr);
+	if (errstr) {
+	  debug2(("error: '%s' is %s\n", &buf[start], errstr));
+	} else if (mode != uim_get_current_mode(g_context)) {
           debug2(("mode change %d\n", mode));
           uim_set_mode(g_context, mode);
           /* callbacks_set_mode(uim_get_current_mode(g_context)); */
@@ -845,7 +858,7 @@ static void main_loop(void)
         focus_in();
       }
 
-      if ((len = read_stdin(buf, sizeof(buf) - 1)) <= 0) {
+      if ((len = read_stdin(buf, sizeof(buf) - 1)) == -1 || len == 0) {
         /* ˤϤʤȻפ */
         return;
       }
@@ -885,11 +898,18 @@ static void main_loop(void)
               t.tv_sec = 0;
               t.tv_usec = g_opt.timeout;
               if (my_select(g_win_in + 1, &fds, &t) > 0) {
-                len += read_stdin(buf + len, sizeof(buf) - len - 1);
-                buf[len] = '\0';
-                debug(("read_again \"%s\"\n", buf));
-                i--;
-                continue;
+                ssize_t nr;
+
+                if ((nr = read_stdin(buf + len, sizeof(buf) - len - 1)) != -1) {
+                  len += nr;
+                  buf[len] = '\0';
+                  debug(("read_again \"%s\"\n", buf));
+                  i--;
+                  continue;
+                } else {
+                  /* XXX: fatal */
+                  return;
+                }
               }
             }
 
@@ -930,7 +950,7 @@ static void main_loop(void)
 
     /* input from pty (child process) */
     if (!g_opt.print_key && FD_ISSET(s_master, &fds)) {
-      if ((len = read(s_master, buf, sizeof(buf) - 1)) <= 0) {
+      if ((len = read(s_master, buf, sizeof(buf) - 1)) == -1 || len == 0) {
         /* ҥץλ */
         return;
       }
@@ -983,14 +1003,14 @@ static void recover_loop(void)
       continue;
     }
     if (FD_ISSET(g_win_in, &fds)) {
-      if ((len = read(g_win_in, buf, sizeof(buf))) <= 0) {
+      if ((len = read(g_win_in, buf, sizeof(buf))) == -1 || len == 0) {
         /* ˤϤʤȻפ */
         return;
       }
       write(s_master, buf, len);
     }
     if (FD_ISSET(s_master, &fds)) {
-      if ((len = read(s_master, buf, sizeof(buf))) <= 0) {
+      if ((len = read(s_master, buf, sizeof(buf))) == -1 || len == 0) {
         /* ҥץλ */
         return;
       }
@@ -1207,7 +1227,7 @@ static void usage(void)
   int max_im_name_len = 0;
 
   uim_init();
-  context = uim_create_context(NULL, get_enc(), NULL, NULL, uim_iconv, commit_cb);
+  context = uim_create_context(NULL, get_enc(), NULL, NULL, NULL, commit_cb);
 
   printf("uim-fep version %s\n", PACKAGE_VERSION);
   printf("Usage: uim-fep [OPTIONS]\n");
