@@ -1,7 +1,7 @@
 /*
  *  $Id:$
  *  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
- *                2005-2007 uim Project http://uim.freedesktop.org/
+ *                2005-2009 uim Project http://code.google.com/p/uim/
  *
  *  All rights reserved.
  *
@@ -31,12 +31,12 @@
  *  SUCH DAMAGE.
  */
 
-#ifndef __dict_cclass_dialog_h_included__
-#define __dict_cclass_dialog_h_included__
+#ifndef UIM_DICT_CCLASS_DIALOG_H
+#define UIM_DICT_CCLASS_DIALOG_H
 
 #include <gtk/gtk.h>
 #include "dict-canna-cclass.h"
 
 gchar *cclass_dialog(gint cclass_type, gint system);
 
-#endif /* __dict_cclass_dialog_h_included__ */
+#endif /* UIM_DICT_CCLASS_DIALOG_H */
