@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+ Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -258,7 +258,7 @@ void CustomPathnameEdit::slotFileDialogFilterSelected( const QString & text )
 
 //----------------------------------------------------------------------------------------
 CustomChoiceCombo::CustomChoiceCombo( struct uim_custom *c, QWidget *parent, const char *name)
-    : QComboBox( parent, name ),
+    : QComboBox( FALSE, parent, name ),
       UimCustomItemIface( c )
 {
     QObject::connect( this, SIGNAL(activated(int)),
