@@ -1,6 +1,6 @@
 ;;; canna-key-custom.scm: Customization variables for canna.scm
 ;;;
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -33,80 +33,80 @@
 
 
 (define-custom-group 'canna-keys1
-		     (_ "Canna key bindings 1")
-		     (_ "long description will be here."))
+		     (N_ "Canna key bindings 1")
+		     (N_ "long description will be here."))
 
 (define-custom-group 'canna-keys2
-		     (_ "Canna key bindings 2")
-		     (_ "long description will be here."))
+		     (N_ "Canna key bindings 2")
+		     (N_ "long description will be here."))
 
 (define-custom-group 'canna-keys3
-		     (_ "Canna key bindings 3")
-		     (_ "long description will be here."))
+		     (N_ "Canna key bindings 3")
+		     (N_ "long description will be here."))
 
 (define-custom-group 'canna-keys4
-		     (_ "Canna key bindings 4")
-		     (_ "long description will be here."))
+		     (N_ "Canna key bindings 4")
+		     (N_ "long description will be here."))
 
 (define-custom 'canna-next-segment-key '(generic-go-right-key)
                '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] next segment")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] next segment")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-prev-segment-key '(generic-go-left-key)
                '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] previous segment")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] previous segment")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-extend-segment-key '("<IgnoreCase><Control>o" "<Shift>right")
                '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] extend segment")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] extend segment")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-shrink-segment-key '("<IgnoreCase><Control>i" "<Shift>left")
                '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] shrink segment")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] shrink segment")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-transpose-as-hiragana-key '("F6" "Muhenkan")
                '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] convert to hiragana")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] convert to hiragana")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-transpose-as-katakana-key '("F7" "Muhenkan")
                '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] convert to katakana")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] convert to katakana")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-transpose-as-halfkana-key '("F8" "Muhenkan")
                '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] convert to halfwidth katakana")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] convert to halfwidth katakana")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-transpose-as-halfwidth-alnum-key '("F10")
 	       '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] convert to halfwidth alphanumeric")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] convert to halfwidth alphanumeric")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-transpose-as-fullwidth-alnum-key '("F9")
 	       '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] convert to fullwidth alphanumeric")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] convert to fullwidth alphanumeric")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-commit-as-opposite-kana-key '("<IgnoreCase><Shift>q")  ;; "Q"
                '(canna-keys1)
 	       '(key)
-	       (_ "[Canna] commit as transposed kana")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] commit as transposed kana")
+	       (N_ "long description will be here"))
 
 ;;
 ;; overriding generic keys
@@ -114,57 +114,57 @@
 (define-custom 'canna-on-key '("<Control>\\" generic-on-key)
                '(canna-keys2)
 	       '(key)
-	       (_ "[Canna] on")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] on")
+	       (N_ "long description will be here"))
 
 ;;(define-custom 'canna-off-key '("l" generic-on-key)
 (define-custom 'canna-off-key '("<Control>\\" generic-off-key)
                '(canna-keys2)
 	       '(key)
-	       (_ "[Canna] off")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] off")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-begin-conv-key '(generic-begin-conv-key)
                '(canna-keys2)
 	       '(key)
-	       (_ "[Canna] begin conversion")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] begin conversion")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-commit-key '(generic-commit-key)
                '(canna-keys2)
 	       '(key)
-	       (_ "[Canna] commit")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] commit")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-cancel-key '(generic-cancel-key)
                '(canna-keys2)
 	       '(key)
-	       (_ "[Canna] cancel")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] cancel")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-next-candidate-key '(generic-next-candidate-key)
                '(canna-keys2)
 	       '(key)
-	       (_ "[Canna] next candidate")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] next candidate")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-prev-candidate-key '(generic-prev-candidate-key)
                '(canna-keys2)
 	       '(key)
-	       (_ "[Canna] previous candidate")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] previous candidate")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-next-page-key '(generic-next-page-key)
              '(canna-keys2)
 	       '(key)
-	       (_ "[Canna] next page of candidate window")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] next page of candidate window")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-prev-page-key '(generic-prev-page-key)
                '(canna-keys2)
 	       '(key)
-	       (_ "[Canna] previous page of candidate window")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] previous page of candidate window")
+	       (N_ "long description will be here"))
 
 ;;
 ;; overriding generic keys (advanced)
@@ -172,56 +172,56 @@
 (define-custom 'canna-beginning-of-preedit-key '(generic-beginning-of-preedit-key)
                '(canna-keys3)
 	       '(key)
-	       (_ "[Canna] beginning of preedit")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] beginning of preedit")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-end-of-preedit-key '(generic-end-of-preedit-key)
                '(canna-keys3)
 	       '(key)
-	       (_ "[Canna] end of preedit")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] end of preedit")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-kill-key '(generic-kill-key)
                '(canna-keys3)
 	       '(key)
-	       (_ "[Canna] erase after cursor")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] erase after cursor")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-kill-backward-key '(generic-kill-backward-key)
                '(canna-keys3)
 	       '(key)
-	       (_ "[Canna] erase before cursor")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] erase before cursor")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-backspace-key '(generic-backspace-key)
                '(canna-keys3)
 	       '(key)
-	       (_ "[Canna] backspace")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] backspace")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-delete-key '(generic-delete-key)
                '(canna-keys3)
 	       '(key)
-	       (_ "[Canna] delete")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] delete")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-go-left-key '(generic-go-left-key)
                '(canna-keys3)
 	       '(key)
-	       (_ "[Canna] go left")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] go left")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-go-right-key '(generic-go-right-key)
                '(canna-keys3)
 	       '(key)
-	       (_ "[Canna] go right")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] go right")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-vi-escape-key '("escape" "<Control>[")
                '(canna-keys3)
 	       '(key)
-	       (_ "[Canna] ESC keys on vi-cooperative mode")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] ESC keys on vi-cooperative mode")
+	       (N_ "long description will be here"))
 
 ;;
 ;; ja advanced
@@ -230,41 +230,41 @@
 (define-custom 'canna-hiragana-key '("<Shift>F6")
 	       '(canna-keys4 mode-transition)
 	       '(key)
-	       (_ "[Canna] hiragana mode")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] hiragana mode")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-katakana-key '("<Shift>F7")
 	       '(canna-keys4 mode-transition)
 	       '(key)
-	       (_ "[Canna] katakana mode")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] katakana mode")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-halfkana-key '("<Shift>F8")
 	       '(canna-keys4 mode-transition)
 	       '(key)
-	       (_ "[Canna] halfwidth katakana mode")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] halfwidth katakana mode")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-halfwidth-alnum-key '("<Shift>F10")
 	       '(canna-keys4 mode-transition)
 	       '(key)
-	       (_ "[Canna] halfwidth alphanumeric mode")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] halfwidth alphanumeric mode")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-fullwidth-alnum-key '("<Shift>F9")
 	       '(canna-keys4 mode-transition)
 	       '(key)
-	       (_ "[Canna] fullwidth alphanumeric mode")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] fullwidth alphanumeric mode")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-kana-toggle-key '()
                '(canna-keys4 advanced)
 	       '(key)
-	       (_ "[Canna] toggle hiragana/katakana mode")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] toggle hiragana/katakana mode")
+	       (N_ "long description will be here"))
 
 (define-custom 'canna-alkana-toggle-key '()
 	       '(canna-keys4 advanced)
 	       '(key)
-	       (_ "[Canna] toggle kana/alphanumeric mode")
-	       (_ "long description will be here"))
+	       (N_ "[Canna] toggle kana/alphanumeric mode")
+	       (N_ "long description will be here"))
