@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -63,7 +63,13 @@
     ((("?"). ())("" "" "?"))
     ((("/"). ())("" "" "/"))
     ((("_"). ())("" "" "_"))
-    ((("yen"). ())("" "" "")) ;; XXX
+    ;; Since ordinary Japanese users press the "yen sign" key on
+    ;; Japanese keyboard in romaji-halfwidth-kana-mode "to input
+    ;; character code 134" rather than "to input yen sign symbol", I
+    ;; changed the fullwidth yen sign with backslash.
+    ;;   -- YamaKen 2007-09-17
+    ;; ((("yen"). ())("" "" "")) ;; XXX
+    ((("yen"). ())("" "" "\\"))
 
     ((("1"). ())("1" "1" "1"))
     ((("2"). ())("2" "2" "2"))
@@ -211,6 +217,7 @@
     ((("d" "y" "o"). ())(("" "" "") ("" "" "")))
 
     ((("n" "n"). ())("" "" ""))
+    ((("n" "'"). ())("" "" ""))
     ((("n"). ())("" "" ""))
 
     ((("n" "a"). ())("" "" ""))
@@ -324,11 +331,11 @@
     ((("l" "t" "u"). ())("" "" ""))
     ((("l" "t" "s" "u"). ())("" "" ""))
 
-    ((("l" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("l" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("l" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("l" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("l" "y" "o"). ())(("" "" "") ("" "" "")))
+    ((("l" "y" "a"). ())("" "" ""))
+    ((("l" "y" "i"). ())("" "" ""))
+    ((("l" "y" "u"). ())("" "" ""))
+    ((("l" "y" "e"). ())("" "" ""))
+    ((("l" "y" "o"). ())("" "" ""))
 
     ((("r" "y" "a"). ())(("" "" "") ("" "" "")))
     ((("r" "y" "i"). ())(("" "" "") ("" "" "")))
@@ -434,6 +441,9 @@
     ((("x" "w" "i"). ())("" "" ""))
     ((("x" "w" "e"). ())("" "" ""))
 
+    ((("w" "y" "i"). ())("" "" ""))
+    ((("w" "y" "e"). ())("" "" ""))
+
     ((("c" "h" "a"). ())(("" "" "") ("" "" "")))
     ((("c" "h" "i"). ())("" "" ""))
     ((("c" "h" "u"). ())(("" "" "") ("" "" "")))
@@ -458,6 +468,58 @@
     ((("g" "w" "e"). ())(("" "" "") ("" "" "")))
     ((("g" "w" "o"). ())(("" "" "") ("" "" "")))
 
+    ((("z" "w" "a"). ())(("" "" "") ("" "" "")))
+    ((("z" "w" "i"). ())(("" "" "") ("" "" "")))
+    ((("z" "w" "u"). ())(("" "" "") ("" "" "")))
+    ((("z" "w" "e"). ())(("" "" "") ("" "" "")))
+    ((("z" "w" "o"). ())(("" "" "") ("" "" "")))
+
+    ;((("n" "w" "a"). ())(("" "" "") ("" "" "")))
+    ;((("n" "w" "i"). ())(("" "" "") ("" "" "")))
+    ;((("n" "w" "u"). ())(("" "" "") ("" "" "")))
+    ;((("n" "w" "e"). ())(("" "" "") ("" "" "")))
+    ;((("n" "w" "o"). ())(("" "" "") ("" "" "")))
+
+    ((("b" "w" "a"). ())(("" "" "̎") ("" "" "")))
+    ((("b" "w" "i"). ())(("" "" "̎") ("" "" "")))
+    ((("b" "w" "u"). ())(("" "" "̎") ("" "" "")))
+    ((("b" "w" "e"). ())(("" "" "̎") ("" "" "")))
+    ((("b" "w" "o"). ())(("" "" "̎") ("" "" "")))
+
+    ((("p" "w" "a"). ())(("" "" "̎") ("" "" "")))
+    ((("p" "w" "i"). ())(("" "" "̎") ("" "" "")))
+    ((("p" "w" "u"). ())(("" "" "̎") ("" "" "")))
+    ((("p" "w" "e"). ())(("" "" "̎") ("" "" "")))
+    ((("p" "w" "o"). ())(("" "" "̎") ("" "" "")))
+
+    ((("m" "w" "a"). ())(("" "" "") ("" "" "")))
+    ((("m" "w" "i"). ())(("" "" "") ("" "" "")))
+    ((("m" "w" "u"). ())(("" "" "") ("" "" "")))
+    ((("m" "w" "e"). ())(("" "" "") ("" "" "")))
+    ((("m" "w" "o"). ())(("" "" "") ("" "" "")))
+
+    ((("y" "w" "a"). ())(("" "" "") ("" "" "")))
+    ((("y" "w" "i"). ())(("" "" "") ("" "" "")))
+    ((("y" "w" "u"). ())(("" "" "") ("" "" "")))
+    ((("y" "w" "e"). ())(("" "" "") ("" "" "")))
+    ((("y" "w" "o"). ())(("" "" "") ("" "" "")))
+
+    ((("r" "w" "a"). ())(("" "" "") ("" "" "")))
+    ((("r" "w" "i"). ())(("" "" "") ("" "" "")))
+    ((("r" "w" "u"). ())(("" "" "") ("" "" "")))
+    ((("r" "w" "e"). ())(("" "" "") ("" "" "")))
+    ((("r" "w" "o"). ())(("" "" "") ("" "" "")))
+
+    ((("d" "'" "i"). ())(("" "" "Î") ("" "" "")))
+    ((("d" "'" "y" "u"). ())(("" "" "Î") ("" "" "")))
+
+    ((("d" "'" "u"). ())(("" "" "Ď") ("" "" "")))
+
+    ((("t" "'" "i"). ())(("" "" "") ("" "" "")))
+    ((("t" "'" "y" "u"). ())(("" "" "") ("" "" "")))
+
+    ((("t" "'" "u"). ())(("" "" "") ("" "" "")))
+ 
     ))
 
 (define ja-rk-rule (append ja-rk-rule-basic ja-rk-rule-additional))
@@ -752,7 +814,9 @@
 	   (list-head lst (- (length lst) (length sub)))
 	   '("")
 	   (ja-join-vu (list-tail lst (+ (- (length lst) (length sub)) 2))))
-	  (if (member "" (cdr sub))
+	  (if (and
+	       sub
+	       (member "" (cdr sub)))
 	      (append
 	       (list-head lst (+ (- (length lst) (length sub)) 1))
 	       (ja-join-vu (cdr sub)))
@@ -774,7 +838,8 @@
   (lambda (sl)
     (if (not (null? sl))
 	(append (list (ja-find-kana-list-from-rule ja-rk-rule-basic (car sl)))
-		(ja-make-kana-str-list (cdr sl))))))
+		(ja-make-kana-str-list (cdr sl)))
+	'())))
 
 (define ja-type-direct	       -1)
 (define ja-type-hiragana	0)
@@ -842,7 +907,13 @@
 
 (define japanese-roma-set-yen-representation
   (lambda ()
-    (set-symbol-value! 'yen ""))) ;; XXX
+    ;; Since ordinary Japanese users press the "yen sign" key on
+    ;; Japanese keyboard in alphanumeric-mode "to input character code
+    ;; 134" rather than "to input yen sign symbol", I changed the
+    ;; fullwidth yen sign with backslash.  -- YamaKen 2007-09-17
+    ;;(set-symbol-value! 'yen "")  ;; XXX
+    (set-symbol-value! 'yen "\\")
+    ))
 
 ;;
 (require "rk.scm")
