@@ -1,6 +1,6 @@
 #!/usr/bin/env gosh
 
-;;; Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -29,7 +29,7 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
-;; These tests are passed at revision 4331 (new repository)
+;; These tests are passed at revision 5329 (new repository)
 
 (use test.unit)
 
@@ -83,21 +83,21 @@
 		   "Hangul (2-bul)"
 		   "2-bul style hangul input method"
 		   ;; replace () with #f for R5RS compliant interpreter
-		   () ;; arg
+		   #f ;; arg
 		   init
-		   () ;; release-handler
-		   () ;; mode-handler
-		   () ;; press-key-handler
-		   () ;; release-key-handler
-		   () ;; reset-handler
-		   () ;; get-candidate-handler
-		   () ;; set-candidate-index-handler
-		   () ;; prop-activate-handler
-		   () ;; input-string-handler
-		   () ;; focus-in-handler
-		   () ;; focus-out-handler
-		   () ;; place-handler
-		   () ;; displace-handler
+		   #f ;; release-handler
+		   #f ;; mode-handler
+		   #f ;; press-key-handler
+		   #f ;; release-key-handler
+		   #f ;; reset-handler
+		   #f ;; get-candidate-handler
+		   #f ;; set-candidate-index-handler
+		   #f ;; prop-activate-handler
+		   #f ;; input-string-handler
+		   #f ;; focus-in-handler
+		   #f ;; focus-out-handler
+		   #f ;; place-handler
+		   #f ;; displace-handler
 		   "hangul")
 		 (uim '(retrieve-im 'hangul2)))
    (uim '(im-set-init-handler! (retrieve-im 'hangul2) init-handler))
@@ -105,15 +105,23 @@
    (assert-true  (uim-bool '(procedure? (im-init-handler
 					 (retrieve-im 'hangul2)))))
    ;; to prevent SEGV on create-context
-   (uim '(define im-update-preedit (lambda arg #f)))
+   (uim '(define im-set-encoding (lambda arg #f)))
+   (uim '(define im-clear-preedit (lambda arg #f)))
    (uim '(define im-pushback-preedit (lambda arg #f)))
+   (uim '(define im-update-preedit (lambda arg #f)))
+   (uim '(define im-update-prop-list (lambda arg #f)))
+   (uim '(define im-clear-mode-list (lambda arg #f)))
+   (uim '(define im-pushback-mode-list (lambda arg #f)))
+   (uim '(define im-update-mode-list (lambda arg #f)))
+   (uim '(define im-update-mode (lambda arg #f)))
 
    (uim '(begin
 	   (create-context 0 #f 'hangul2)
 	   #f))
    (uim '(begin
-	   (define test-context (find-context 0))
+	   (define test-context (assv 0 context-list))
 	   #f))
+
    (assert-equal 'hangul2
 		 (uim '(im-name (context-im test-context))))
    (assert-equal "hangul"
