@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -573,7 +573,7 @@ main(int argc, char **argv)
 {
     const char *locale;
 
-    printf("UIM-XIM bridge. Now supporting multiple locales.\n");
+    printf("uim <-> XIM bridge. Supporting multiple locales.\n");
 
     get_runtime_env();
 
