@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -30,8 +30,8 @@
   SUCH DAMAGE.
 */
 
-#ifndef _util_h_included_
-#define _util_h_included_
+#ifndef UIM_XIM_UTIL_H
+#define UIM_XIM_UTIL_H
 
 // fd dispatch
 #define READ_OK 1
